/*
	*
	* This file is a part of libpaperde.
	* Library for handling various services related to Paper Desktop
	* Copyright 2020 CuboCore Group
	*

	*
	* This file was originally a part of CoreApps project (https://gitlab.com/cubocore/coreapps)
	* Suitable modifications have been done to meet the needs of Paper Desktop.
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#pragma once

#include <QObject>
#include <QFont>
#include <QRect>

#include "paperglobals.h"

class QDBusInterface;

class LIBPAPER_DLLSPEC papersettings : public QObject {
	Q_OBJECT

public:
	/*
	 * App will be one of the various apps:
	 * papershell, bluetooth, network, power, and so on...
	 */
	static papersettings *instance(QString app = QString("papershell"));

	static QString paperdeDbusPath();
	static QString paperdeDbusService();

	struct PaperProxy {
		QVariant value;

		operator int() const {

			return value.toInt();
		}

		operator uint() const {

			return value.toUInt();
		}

		operator float() const {

			return value.toReal();
		}

		operator double() const {

			return value.toDouble();
		}

		operator bool() const {

			return value.toBool();
		}

		operator QString() const {

			return value.toString();
		}

		operator QStringList() const {

			return value.value<QStringList>();
		}

		operator QSize() const {

			return value.toSize();
		}

		operator QFont() const {

			return value.value<QFont>();
		}

		operator QRect() const {

			return value.toRect();
		}
	};

	void reload();

	PaperProxy value(const QString &key);
	QVariant rawValue(const QString &key);
	void setValue(const QString &key, QVariant value);

signals:
	void settingsChanged(QString appName, QString key, QVariant value);

private:
	/* @app -> Settings of a specific app: Like Files, Term, Shell, etc... */
	papersettings(QString app);

	QString mAppName;
	QDBusInterface *iface;

	QSettings *fallback;

	static QMap<QString, papersettings *> mSettingsMap;
	static papersettings *mSettings;

private slots:
	void emitSettingsChanged(QVariantMap);
};
