/*
	*
	* This file is a part of libpapershell-wl.
	* Library for handling various services related to Paper Shell.
	* Copyright 2020 CuboCore Group
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#pragma once

#include "qwayland-wlr-layer-shell-unstable-v1.h"
#include "layershell.h"

class layerShellPrivate : public QtWayland::zwlr_layer_shell_v1 {

public:
	layerShellPrivate(struct ::zwlr_layer_shell_v1 *lShell, layerShell *ptr);
	~layerShellPrivate();

	struct ::zwlr_layer_surface_v1 *createlayerSurface(QWindow *window, layerShell::LayerType layer, const QString &layer_namespace);

protected:
	Q_DISABLE_COPY(layerShellPrivate)
	Q_DECLARE_PUBLIC(layerShell)

	layerShell *q_ptr;
};

class layerSurfacePrivate : public QtWayland::zwlr_layer_surface_v1 {

public:
	layerSurfacePrivate(QWindow *window, layerShell *lShell, layerSurface *ptr);
	~layerSurfacePrivate();

	void initialize(layerShell::LayerType lt, const QString &layerNamespace);
	void apply();

protected:
	void zwlr_layer_surface_v1_configure(uint32_t serial, uint32_t width, uint32_t height) override;
	void zwlr_layer_surface_v1_closed() override;

	Q_DISABLE_COPY(layerSurfacePrivate)
	Q_DECLARE_PUBLIC(layerSurface)

	layerSurface *q_ptr;

private:
	QWindow *m_window;
	layerShell *m_layerShell;

	QSize m_surfaceSize;
	layerSurface::SurfaceAnchors m_anchors;
	int m_exclusiveZone;
	QMargins m_margins;
	bool m_keyboardInteractivity;

	QSize m_surfacePendingSize;
};
