/*
	*
	* This file is a part of libpaper.
	* Library for handling various services related to Paper Shell.
	* Copyright 2020 CuboCore Group
	*

	*
	* This program is free software; you can redistribute it and/or modify
	* it under the terms of the GNU General Public License as published by
	* the Free Software Foundation; either version 3 of the License, or
	* (at your option) any later version.
	*

	*
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	*

	*
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	* MA 02110-1301, USA.
	*
*/

#pragma once

#include <QObject>

#include "paperglobals.h"

class toplevelHandle;
class toplevelHandlePrivate;
class toplevelManagerPrivate;

struct wl_seat;
struct wl_array;
struct wl_output;
struct zwlr_foreign_toplevel_handle_v1;
struct zwlr_foreign_toplevel_manager_v1;

typedef QList<toplevelHandle *> toplevelHandleList;

class LIBPAPER_DLLSPEC toplevelManager : public QObject {
	Q_OBJECT

public:
	toplevelManager(zwlr_foreign_toplevel_manager_v1 *tlManager, wl_seat *seat, QObject *parent = nullptr);
	~toplevelManager();

	toplevelHandleList tlHandleList() const;
	void activateHandle(toplevelHandle *handle);
	void showDesktop();
	void minimizeAll();

signals:
	void newToplevelHandle(toplevelHandle *handle);

protected:
	Q_DECLARE_PRIVATE(toplevelManager)
	toplevelManagerPrivate *d_ptr;

private:
	void closeHandle(toplevelHandle *tlHandle);
};

class LIBPAPER_DLLSPEC toplevelHandle : public QObject {
	Q_OBJECT

public:
	enum ToplevelState {
		None = (1 << 0),
		Activated = (1 << 1),
		Maximized = (1 << 2),
		Minimized = (1 << 3)
	};
	Q_DECLARE_FLAGS(ToplevelStates, ToplevelState)

	toplevelHandle(zwlr_foreign_toplevel_handle_v1 *handle, QObject *parent = nullptr);
	~toplevelHandle();

	QString appID() const;
	QString appTitle() const;
	toplevelHandle::ToplevelStates appStates() const;

	void activate(wl_seat *seat);
	void setMaximized();
	void unsetMaximized();
	void setMinimized();
	void unsetMinimized();
	void close();

signals:
	void handleUpdated(toplevelHandle *handle);
	void handleClosed(toplevelHandle *handle);

protected:
	Q_DECLARE_PRIVATE(toplevelHandle)
	toplevelHandlePrivate *d_ptr;
};

Q_DECLARE_OPERATORS_FOR_FLAGS(toplevelHandle::ToplevelStates)
